#!/usr/bin/python3

import argparse


def write_ucs2(a, description):
    l = len(a)

    print("/* %s */" % description)
    print("#define AO_%s_LEN 0x%02x" % (description, l * 2 + 2))
    print('#define AO_%s_STRING "%s"' % (description, a))
    print("#define AO_%s_UCS2" % description, end="")
    for i in range(l):
        c = a[i]
        if i > 0:
            print(",", end="")
        if 0x20 <= ord(c) and ord(c) < 128:
            print(" '%c', 0" % c, end="")
        else:
            print(" LE_WORD(0x%04x)" % ord(c), end="")
    print("\n")


def write_string(a, description):
    print("/* %s */" % description)
    print('#define AO_%s_STRING "%s"' % (description, a))


def write_int(a, description):
    print("/* %s */" % description)
    print("#define AO_%s_NUMBER %d\n" % (description, a))


def write_hex(a, description):
    print("/* %s */" % description)
    print("#define AO_%s_NUMBER 0x%04x\n" % (description, a))


def auto_int(x):
    return int(x, 0)


def make_product():
    manufacturer = "keithp.com"
    product = "MetroSnek"
    version = "0.0"
    output = ""
    serial = 1
    user_argind = 0
    id_vendor = 0xFFFE
    id_product = 0x000A
    parse = argparse.ArgumentParser()
    parse.add_argument(
        "-m", "--manufacturer", help="manufacturer name", default=manufacturer
    )
    parse.add_argument("-p", "--product", help="product name", default=product)
    parse.add_argument(
        "-V", "--id_vendor", type=auto_int, help="vendor id number", default=id_vendor
    )
    parse.add_argument(
        "-i",
        "--id_product",
        type=auto_int,
        help="product id number",
        default=id_product,
    )
    parse.add_argument("-v", "--version", help="version string", default=version)
    parse.add_argument(
        "-s", "--serial", type=auto_int, help="serial number", default=serial
    )
    args = parse.parse_args()
    if args.manufacturer:
        manufacturer = args.manufacturer
    if args.product:
        product = args.product
    if args.id_vendor:
        id_vendor = args.id_vendor
    if args.id_product:
        id_product = args.id_product
    if args.version:
        version = args.version
    if args.serial:
        serial = args.serial
    write_ucs2(manufacturer, "iManufacturer")
    write_ucs2(product, "iProduct")
    write_ucs2("%06d" % serial, "iSerial")
    write_int(serial, "iSerial")
    write_hex(id_product, "idProduct")
    write_hex(id_vendor, "idVendor")
    write_string(version, "iVersion")


make_product()



#
# PID line bug
#

# This needs to be determined experimentally and
# should be above the value of 'black'

black = 0.77
white = 0.06
threshold = (black + white) / 2


# Provide an 'error' value between -1 and 1 indicating
# how far off we are from the line
def measure():
    return (threshold - read(A1)) / threshold


# Experimentally determined PID constants
Kp = 0.6
Ki = 0.004
Kd = 1

integral = 0
previous_error = 0


# Compute control value based on measured error
def process():
    global integral, previous_error

    # Measure the current error
    error = measure()

    # Integrate error into the I term
    integral += error

    # Take the difference into the D term
    derivative = error - previous_error

    # Save the current error for use next time
    previous_error = error

    # Return the computed PID output, combining
    # the current error, integral and derivative terms
    return Kp * error + Ki * integral + Kd * derivative


# Set motor speed and direction using
# values from -1 to 1
def set_speed(m, val):
    talkto(m)
    if val < 0:
        setright()
        setpower(-val)
    else:
        setleft()
        setpower(val)


#
# Set speeds of both motors using  values from -2 to 2.
#
# Values from -2 to -1 turn left motor full forward
# and right motor from full reverse to stop
#
# Values from -1 to 0 turn left motor full forward
# and right motor from stop to full forward
#
# Values from 0 to 1 turn right motor full forward
# and left motor from full forward to stop
#
# Values from 1 to 2 turn right motor full forward
# and left motor from stop to full reverse
def set_speeds(value):
    if value < 0:
        # reduce speed of left motor
        set_speed(M3, 1 + value)
        set_speed(M1, -1)
    else:
        # reduce speed of right motor
        set_speed(M1, value - 1)
        set_speed(M3, 1)


# Initialize
def init():
    global integral, previous_error, Ki_run

    # Turn both motors on, but set the power to zero
    talkto(M1)
    setpower(0)
    on()
    talkto(M3)
    setpower(0)
    on()

    # Initialize PID loop values
    integral = 0
    previous_error = 0
    Ki_run = 0


# compute control value, update motor speeds
def loop():
    init()
    while True:
        # Compute desired motor setting
        value = process()

        # Set the motors
        set_speeds(value)


loop()



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#

# so many options to pick here; just do a few simple tests

assert "%d" % 3 == "3"
assert "%d %%" % 3 == "3 %"
assert "%d" % 3.1 == "3"
assert "%f" % 3.1 == "3.100000"
assert "%g" % 3.1 == "3.1"
assert "%e" % 3.1 == "3.100000e+00"
assert "%s" % "hello" == "hello"
assert "%r" % "hello" == "'hello'"
assert ("%d" + "x" * 300 + "%d") % (
    12,
    34,
) == "12xxxxxxxxxx" + "xx" * 140 + "xxxxxxxxxx34"
assert "%r" % {1: "hello", 2: "world"} == "{1: 'hello', 2: 'world'}"



#!/usr/bin/snek
#
# Imagine a circle inscribed in a square. The ratio of their areas is
# the same as the ratio of a random point in the square also being in
# the circle
#
# We'll use a circle of radius 1, which means the square is 2x2. The
# circle has area π, the square has area 4:
#
# circle / square = π / 4 = inside / tries
#
# π = inside / tries * 4
#

import random
import time


def pi(outer):
    start = time.monotonic()
    inner = 100
    inside = 0
    for o in range(outer):
        for i in range(inner):
            # Pick a random point in the square. This point will have
            # coordinates from (-1,-1) to (1,1)
            x = random.random() * 2 - 1
            y = random.random() * 2 - 1
            # Check to see if the point is inside the circle
            if x * x + y * y <= 1:
                inside += 1
    print("inside %d pi %g" % (inside, inside / (outer * inner) * 4))
    stop = time.monotonic()
    print("time %f" % (stop - start))


pi(100)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#
# Test arithmetic operations
#

from math import *

if 2 + 3 != 5:
    exit(1)
if 3 + 2 != 5:
    exit(1)
if 2 - 3 != -1:
    exit(1)
if 3 - 2 != +1:
    exit(1)
if 2 * 3 != 6:
    exit(1)
if 2 * (3 + 4) != 14:
    exit(1)
if (2 * 3) + 4 != 10:
    exit(1)
if 2 * 3 + 4 != 10:
    exit(1)
if 2**3**4 == 4096:
    exit(1)
if (2**3) ** 4 != 4096:
    exit(1)
if 7.8 // 2.2 != 3.0:
    exit(1)
if 8.7 // 2.2 != 3.0:
    exit(1)
if 8.8 // 2.2 != 4.0:
    exit(1)
if -6.7 // 2.2 != -4.0:
    exit(1)
if -6.6 // 2.2 != -3.0:
    exit(1)
if not isclose(7.8 % 2.2, 1.2):
    exit(1)
if not isclose(-7.8 % 2.2, 1.0):
    exit(1)
inf = float("inf")

vals = (-inf, -2, -1, 0, 1, 2, inf)

for first in range(0, len(vals)):
    assert vals[first] == vals[first] + 0

    for second in range(first, len(vals)):
        assert vals[first] <= vals[second] + 0

        assert vals[second] >= vals[first] + 0

    for second in range(first + 1, len(vals)):
        assert vals[first] < vals[second] + 0

        assert vals[second] > vals[first] + 0

    for second in range(0, len(vals)):
        if second != first:
            assert vals[first] != vals[second] + 0



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#
# tuples don't have in-place operation,
# so += makes a new tuple
#

a = ()
b = a
a += (1, 2)

if a != (1, 2):
    exit(1)
if b != ():
    exit(1)

#
# lists have in-place operation,
# so += does not make a new list
# while a + b does make a new list
#

a = []
b = a
c = a + [1, 2]
a += [1, 2]

if a != [1, 2]:
    exit(1)
if b != [1, 2]:
    exit(1)
if c != [1, 2]:
    exit(1)

a[0] = 3

if a != [3, 2]:
    exit(1)
if b != [3, 2]:
    exit(1)
if c != [1, 2]:
    exit(1)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
def g(i):
    if i == 0:
        return 100
    elif i == 1:
        return 101
    elif i == 2:
        return 102
    else:
        return 103


if not (
    g(0) == 100 and g(1) == 101 and g(2) == 102 and g(3) == 103 and g("hello") == 103
):
    exit(1)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#

import sys
import argparse


def fprint(msg="", end="\n", file=sys.stdout):
    file.write(msg)
    file.write(end)


builtin_id = 1


class SnekBuiltin:
    def __init__(self, name, param, value, globals=None, condition=None):
        self.name = name
        self.value = None
        self.init = None
        self.alias = None
        self.condition = condition
        self.id = -1
        self.full_name = name.replace(".", "_")
        self.global_alias = False
        if param[0].isalpha():
            self.keyword = param
            self.nformal = -3
        else:
            self.keyword = False
            self.nformal = int(param)
        if value and value[0] == "=":
            if self.nformal == -2:
                self.init = value[1:]
            else:
                self.alias = value[1:]
        else:
            self.value = value
            for g in globals:
                if name.startswith(g + "."):
                    self.alias = name
                    self.global_alias = True
                    self.name = name.removeprefix(g + ".")
                    self.full_name = self.name
                    break

    def __eq__(self, other):
        return self.name == other.name

    def is_name(self):
        return self.nformal == -2 and self.value is None

    def is_value(self):
        return self.nformal == -2 and self.value is not None

    def is_init(self):
        return self.nformal == -2 and self.init is not None

    def is_func(self):
        return self.nformal >= -1

    def kind_order(self):
        if self.is_func():
            return 0
        elif self.keyword:
            return 2
        else:
            return 1

    def value_order(self, other):
        if self.value is None and other.value is not None:
            return 1
        if other.value is None and self.value is not None:
            return -1
        return 0

    def __lt__(self, other):
        o = self.kind_order() - other.kind_order()
        if o < 0:
            return True
        elif o > 0:
            return False
        o = self.value_order(other)
        if o != 0:
            return o < 0
        return self.name < other.name

    def snek_name(self):
        return self.name

    def cpp_name(self):
        return "SNEK_BUILTIN_%s" % self.full_name

    def func_name(self):
        if self.is_value():
            if self.value[0].isdigit():
                return "(snek_poly_t)(float)%s" % self.value
            return self.value
        name = self.name
        if self.alias is not None:
            name = self.alias
        return "snek_builtin_%s" % (name.replace(".", "_"))

    def func_field(self):
        if self.is_value():
            return ".value"
        elif self.nformal == -1:
            return ".funcv"
        return ".func%d" % self.nformal

    def set_id(self):
        global builtin_id
        if not self.keyword and self.id == -1:
            self.id = builtin_id
            builtin_id += 1

    def __str__(self):
        return (
            "builtin(name='%s', value='%r', init='%r', alias='%s', full_name='%s')"
            % (self.name, self.value, self.init, self.alias, self.full_name)
        )


headers = []
builtins = []
globals = []
aliases = False


def add_builtin(name, id, value, condition):
    global builtins, globals, aliases
    builtin = SnekBuiltin(name, id, value, globals, condition)
    for b in builtins:
        if b.name == builtin.name:
            print("%s: skipping duplicate" % name)
            return
    builtins += [builtin]
    if aliases and builtin.global_alias:
        builtin = SnekBuiltin(name, id, value, [], condition)
        builtins += [builtin]


def load_builtins(filename):
    global headers
    current_condition = None
    f = open(filename)
    for line in f.readlines():
        line = line.rstrip()
        if len(line) == 0:
            pass
        elif line.startswith("#ifdef"):
            current_condition = line.split(" ")[1].strip()
        elif line.startswith("#endif") and current_condition:
            current_condition = None
        elif line[0] == "#":
            if len(line) > 1 and line[1] != " ":
                headers += [line]
        else:
            bits = line.split(",")
            value = None
            if len(bits) > 2:
                value = ",".join(bits[2:]).strip()
            add_builtin(bits[0].strip(), bits[1].strip(), value, current_condition)


def dump_headers(fp):
    for line in headers:
        fprint("%s" % line, file=fp)


init_header = """#define snek_init() {\\
	snek_globals = snek_alloc(sizeof (snek_frame_t));\\
	snek_globals->prev = SNEK_OFFSET_NONE;\\
	snek_globals->code = SNEK_OFFSET_NONE;\\"""


def dump_init(fp):
    use_list_build = False
    use_string_build = False
    fprint(init_header, file=fp)
    for name in sorted(builtins):
        if name.is_init():
            if "snek_list_build" in name.init:
                use_list_build = True
            if "snek_string_build" in name.init:
                use_string_build = True
            fprint("    {\\", file=fp)
            fprint("        snek_stack_push((%s));\\" % name.init, file=fp)
            fprint(
                "        *snek_id_ref(%s, true) = snek_stack_pop();\\"
                % name.cpp_name(),
                file=fp,
            )
            fprint("    }\\", file=fp)
    fprint("}", file=fp)
    if use_list_build:
        fprint("#define SNEK_LIST_BUILD", file=fp)
    if use_string_build:
        fprint("#define SNEK_STRING_BUILD", file=fp)


def dump_max_len(fp):
    max_len = 0
    for name in builtins:
        if len(name.name) > max_len:
            max_len = len(name.name)
    fprint("#define SNEK_BUILTIN_NAMES_MAX_LEN %d" % max_len, file=fp)
    fprint("#define SNEK_BUILTIN_NAMES_MAX_ARGS %d" % max_args(), file=fp)


def set_ids():
    for name in sorted(builtins):
        name.set_id()


def dump_names(fp):
    fprint(
        "static CONST uint8_t snek_builtin_names[] = {",
        file=fp,
    )
    total = 0
    for name in sorted(builtins):
        if name.condition:
            fprint("#ifdef %s" % name.condition, file=fp)

        if name.keyword:
            fprint("\t%s, " % name.keyword, end="", file=fp)
            total += 1
        else:
            fprint("\t", end="", file=fp)
        for c in name.name:
            fprint("'%c', " % c, end="", file=fp)
            total += 1
        fprint("0,", file=fp)

        if name.condition:
            fprint("#endif", file=fp)

        total += 1
    fprint("};", file=fp)
    fprint("#define SNEK_BUILTIN_NAMES_SIZE %d" % total, file=fp)


def trim_mu(n):
    if n.find(".") >= 0:
        return n[: n.find(".")]
    return n


mu_skip = ["len", "print", "end", "False", "True", "ord", "chr", "None"]


def dump_mu(fp):
    all_names = {}
    for name in sorted(builtins):
        if not name.keyword and not name.name in mu_skip:
            all_names[trim_mu(name.name)] = True

    for name in sorted(all_names.keys()):
        fprint("        '%s'," % name, file=fp)


def max_args():
    max = 0
    for name in sorted(builtins):
        if name.keyword or not name.is_func():
            continue
        if name.nformal > max:
            max = name.nformal
    return max


def dump_decls(fp):
    for name in sorted(builtins):
        if not name.is_func():
            continue
        fprint("extern snek_poly_t", file=fp)
        fprint("%s(" % name.func_name(), file=fp, end="")
        if name.nformal == -1:
            fprint(
                "uint8_t nposition, uint8_t nnamed, snek_poly_t *args", end="", file=fp
            )
        elif name.nformal == 0:
            fprint("void", end="", file=fp)
        else:
            for a in range(name.nformal):
                fprint("snek_poly_t a%d" % a, end="", file=fp)
                if a < name.nformal - 1:
                    fprint(", ", end="", file=fp)
        fprint(");", file=fp)
        fprint(file=fp)


def dump_builtins(fp):
    fprint("CONST snek_builtin_t snek_builtins[] = {", file=fp)

    for name in sorted(builtins):
        if name.keyword:
            continue

        if name.is_name():
            continue

        fprint("\t[%s - 1] = {" % name.cpp_name(), file=fp)
        if name.is_func():
            fprint("\t\t.nformal=%d," % name.nformal, file=fp)
        fprint("\t\t%s = %s," % (name.func_field(), name.func_name()), file=fp)
        fprint("\t},", file=fp)
    fprint("};", file=fp)


def dump_cpp(fp):
    marked_funcs = False
    marked_values = False
    for name in sorted(builtins):
        if name.keyword:
            continue
        if name.is_value() and not marked_funcs:
            fprint("#define SNEK_BUILTIN_MAX_FUNC %d" % name.id, file=fp)
            marked_funcs = True
        if name.is_name() and not marked_values:
            fprint("#define SNEK_BUILTIN_MAX_BUILTIN %d" % name.id, file=fp)
            marked_values = True
        fprint("#define %s %d" % (name.cpp_name(), name.id), file=fp)

    fprint("#define SNEK_BUILTIN_END %d" % (builtin_id), file=fp)


def builtin_main():
    global globals, aliases

    parser = argparse.ArgumentParser(description="Construct Snek builtin data.")
    parser.add_argument(
        "builtins", metavar="F", nargs="+", help="input files describing builtins"
    )
    parser.add_argument("-o", "--output", dest="output", help="output file")
    parser.add_argument(
        "-m", "--mu", action="store_true", help="output just non-keyword constants"
    )
    parser.add_argument(
        "-a", "--alias", action="store_true", help="--globals are also namespaced"
    )
    parser.add_argument(
        "-g",
        "--global",
        action="append",
        dest="globals",
        help="remove this leading namespace",
    )

    args = parser.parse_args()

    if args.globals:
        globals = args.globals

    aliases = args.alias

    for b in args.builtins:
        load_builtins(b)

    fp = sys.stdout

    if args.output:
        fp = open(args.output, mode="w")

    if args.mu:
        dump_mu(fp)
        return

    fprint("#ifdef SNEK_BUILTIN_DATA", file=fp)
    fprint("#undef SNEK_BUILTIN_DATA", file=fp)

    set_ids()

    dump_names(fp)

    fprint(file=fp)

    max_formals = max_args()

    dump_builtins(fp)

    fprint(file=fp)

    fprint("#else /* SNEK_BUILTIN_DATA */", file=fp)

    fprint("#ifdef SNEK_BUILTIN_DECLS", file=fp)
    fprint("#undef SNEK_BUILTIN_DECLS", file=fp)
    dump_decls(fp)
    fprint("#else /* SNEK_BUILTIN_DECLS */", file=fp)

    dump_max_len(fp)

    dump_headers(fp)

    dump_init(fp)

    dump_cpp(fp)

    fprint("#endif /* SNEK_BUILTIN_DECLS */", file=fp)
    fprint("#endif /* SNEK_BUILTIN_DATA */", file=fp)


builtin_main()



#
# Copyright © 2020 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

ops = ("<", "<=", "==", "!=", ">=", ">")
n_vals = (0, 1, 2)
s_vals = ("a", "b", "c")


def perm(vals, ops):
    for f_op in ops:
        for s_op in ops:
            print("if %r %s %r %s %r:" % (vals[0], f_op, vals[1], s_op, vals[2]))
            print(
                ' print("assert %r %s %r %s %r")'
                % (vals[0], f_op, vals[1], s_op, vals[2])
            )
            print("else:")
            print(
                ' print("assert not(%r %s %r %s %r)")'
                % (vals[0], f_op, vals[1], s_op, vals[2])
            )


perm(n_vals, ops)
perm(s_vals, ops)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#

from random import *

servo = M4
motor = M1
front = A1
back = A2


def see(sensor):
    return read(sensor) > 0.1


def setservo(v):
    talkto(servo)
    if v < 0:
        setleft()
        v = -v
    else:
        setright()
    setpower(v)
    on()


def go(t, d):
    talkto(motor)
    s = front
    if d:
        setright()
        s = front
    else:
        setleft()
        s = back
    setpower(1)
    onfor(t)


def straight(t, d):
    setservo(0)
    go(t, d)


def left(t, d):
    setservo(1)
    go(t, d)


def right(t, d):
    setservo(-1)
    go(t, d)


def bounce():
    straight(randrange(10) / 10 + 1, True)
    right(randrange(10) / 10 + 1, False)
    left(randrange(10) / 10 + 1, True)
    straight(randrange(10) / 10 + 1, False)


while True:
    bounce()



import os
import pexpect
import serial
import sys
import threading
import time

try:
    import IDF
except ImportError:
    test_fw_path = os.getenv("TEST_FW_PATH")
    if test_fw_path and test_fw_path not in sys.path:
        sys.path.insert(0, test_fw_path)
    import IDF

import Utility


class CustomProcess(object):
    def __init__(self, cmd, logfile):
        self.f = open(logfile, "wb")
        self.p = pexpect.spawn(cmd, timeout=60, logfile=self.f)

    def __enter__(self):
        return self

    def close(self):
        self.p.terminate(force=True)

    def __exit__(self, type, value, traceback):
        self.close()
        self.f.close()


class OCDProcess(CustomProcess):
    def __init__(self, proj_path):
        cmd = "openocd -f interface/ftdi/esp32_devkitj_v1.cfg -f board/esp-wroom-32.cfg"
        log_file = os.path.join(proj_path, "openocd.log")
        super(OCDProcess, self).__init__(cmd, log_file)
        i = self.p.expect_exact(
            ["Info : Listening on port 3333 for gdb connections", "Error:"]
        )
        if i == 0:
            Utility.console_log("openocd is listening for gdb connections")
        else:
            raise RuntimeError("openocd initialization has failed")

    def close(self):
        try:
            self.p.sendcontrol("c")
            self.p.expect_exact("shutdown command invoked")
        except Exception:
            Utility.console_log("openocd needs to be killed", "O")
            super(OCDProcess, self).close()


class GDBProcess(CustomProcess):
    def __init__(self, proj_path, elf_path):
        cmd = "xtensa-esp32-elf-gdb -x {} --directory={} {}".format(
            os.path.join(proj_path, ".gdbinit.ci"),
            os.path.join(proj_path, "main"),
            elf_path,
        )
        log_file = os.path.join(proj_path, "gdb.log")
        super(GDBProcess, self).__init__(cmd, log_file)
        self.p.sendline(
            ""
        )  # it is for "---Type <return> to continue, or q <return> to quit---"
        i = self.p.expect_exact(
            ["Thread 1 hit Temporary breakpoint 2, app_main ()", "Load failed"]
        )
        if i == 0:
            Utility.console_log("gdb is at breakpoint")
        elif i == 1:
            raise RuntimeError("Load has failed. Please examine the logs.")
        else:
            Utility.console_log("i = {}".format(i))
            Utility.console_log(str(self.p))
            # This really should not happen. TIMEOUT and EOF failures are exceptions.
            raise RuntimeError(
                "An unknown error has occurred. Please examine the logs."
            )

        self.p.expect_exact("(gdb)")

    def close(self):
        try:
            self.p.sendline("q")
            self.p.expect_exact("Quit anyway? (y or n)")
            self.p.sendline("y")
            self.p.expect_exact("Ending remote debugging.")
        except Exception:
            Utility.console_log("gdb needs to be killed", "O")
            super(GDBProcess, self).close()

    def break_till_end(self):
        self.p.sendline("b esp_restart")
        self.p.sendline("c")
        self.p.expect_exact("Thread 1 hit Breakpoint 3, esp_restart ()")


class SerialThread(object):
    def run(self, log_path, exit_event):
        with serial.Serial("/dev/ttyUSB1", 115200) as ser, open(log_path, "wb") as f:
            while True:
                f.write(ser.read(ser.in_waiting))
                if exit_event.is_set():
                    break
                time.sleep(1)

    def __init__(self, log_path):
        self.exit_event = threading.Event()
        self.t = threading.Thread(
            target=self.run,
            args=(
                log_path,
                self.exit_event,
            ),
        )
        self.t.start()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.exit_event.set()
        self.t.join(60)
        if self.t.is_alive():
            Utility.console_log("The pyserial thread is still alive", "O")


@IDF.idf_example_test(env_tag="test_jtag_arm")
def test_examples_loadable_elf(env, extra_data):
    idf_path = os.environ["IDF_PATH"]
    rel_project_path = os.path.join("examples", "get-started", "hello_world")
    proj_path = os.path.join(idf_path, rel_project_path)
    example = IDF.Example(rel_project_path)
    sdkconfig = example.get_sdkconfig()
    elf_path = os.path.join(
        example.get_binary_path(rel_project_path), "hello-world.elf"
    )
    esp_log_path = os.path.join(proj_path, "esp.log")

    assert sdkconfig["CONFIG_IDF_TARGET_ESP32"] == "y", "Only ESP32 target is supported"
    assert (
        sdkconfig["CONFIG_APP_BUILD_TYPE_ELF_RAM"] == "y"
    ), "ELF should be built with CONFIG_APP_BUILD_TYPE_ELF_RAM"

    with SerialThread(esp_log_path):
        with OCDProcess(proj_path), GDBProcess(proj_path, elf_path) as gdb:
            gdb.break_till_end()

    if pexpect.run(
        'grep "Restarting now." {}'.format(esp_log_path), withexitstatus=True
    )[1]:
        raise RuntimeError("Expected output from ESP was not received")


if __name__ == "__main__":
    test_examples_loadable_elf()



#!/usr/bin/python3
#
# Copyright © 2021 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#

#
# Make sure that when nan is compared with any number
# the result is false for any op other than !=
#

nan = float("nan")
inf = float("inf")

for x in (nan, -nan, -inf, inf, -1, 0, 1):
    assert not (nan == x)
    assert not (x == nan)

    assert nan != x
    assert x != nan

    assert not (nan > x)
    assert not (x > nan)

    assert not (nan < x)
    assert not (x < nan)

    assert not (nan >= x)
    assert not (x >= nan)

    assert not (nan <= x)
    assert not (x <= nan)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#


def slice_model(start, end, step, len):
    indices = []

    if step == ():
        step = 1

    ret = []

    if step > 0:
        # default start is 0
        if start == ():
            start = 0

        # start < 0 is from end

        elif start < 0:
            start = len + start

        # default end is len
        if end == ():
            end = len

        # end < 0 is from end
        elif end < 0:
            end = len + end

        # bounds check start
        if start < 0:
            start = 0

        val = start

        while val < end:
            if 0 <= val and val < len:
                ret += [val]
            val += step

    else:
        # default start is len - 1
        if start == ():
            start = len - 1
        elif start < 0:
            start = len + start

        # default end is -1
        if end == ():
            end = -1
        elif end < 0:
            end = len + end

        if start >= len:
            start = len - 1

        val = start
        while val > end:
            if 0 <= val and val < len:
                ret += [val]
            val += step

    return ret


def slice_string(value, start, end, step):
    indices = slice_model(start, end, step, len(value))

    ret = ""
    for i in indices:
        ret += value[i]
    return ret


def slice_list(value, start, end, step):
    indices = slice_model(start, end, step, len(value))

    ret = []
    for i in indices:
        ret += [value[i]]
    return ret


def slice_tuple(value, start, end, step):
    indices = slice_model(start, end, step, len(value))

    ret = ()
    for i in indices:
        ret += (value[i],)
    return ret


s = "hello"
l = [1, 2, 3, 4, 5]
t = (1, 2, 3, 4, 5)


def check(model, impl, mess):
    if model != impl:
        print("fail: %s (model %r impl %r)" % (mess, model, impl))
        exit(1)


# Test lots of combinations of ranges

for start in range(-5, 5):
    for end in range(-5, 5):
        for step in range(-5, 5):
            if step == 0:
                continue

            check(slice_string(s, (), (), ()), s[::], "[::]")

            check(slice_string(s, start, (), ()), s[start::], "[%d::]" % (start))

            check(slice_string(s, (), end, ()), s[:end:], "[:%d:]" % (end))

            check(
                slice_string(s, start, end, ()),
                s[start:end:],
                "[%d:%d:]" % (start, end),
            )

            check(slice_string(s, (), (), step), s[::step], "[::%d]" % (step))

            check(
                slice_string(s, start, (), step),
                s[start::step],
                "[%d::%d]" % (start, step),
            )

            check(
                slice_string(s, (), end, step), s[:end:step], "[:%d:%d]" % (end, step)
            )

            check(
                slice_string(s, start, end, step),
                s[start:end:step],
                "[%d:%d:%d]" % (start, end, step),
            )

            check(slice_list(l, (), (), ()), l[::], "[::]")

            check(slice_list(l, start, (), ()), l[start::], "[%d::]" % (start))

            check(slice_list(l, (), end, ()), l[:end:], "[:%d:]" % (end))

            check(
                slice_list(l, start, end, ()), l[start:end:], "[%d:%d:]" % (start, end)
            )

            check(slice_list(l, (), (), step), l[::step], "[::%d]" % (step))

            check(
                slice_list(l, start, (), step),
                l[start::step],
                "[%d::%d]" % (start, step),
            )

            check(slice_list(l, (), end, step), l[:end:step], "[:%d:%d]" % (end, step))

            check(
                slice_list(l, start, end, step),
                l[start:end:step],
                "[%d:%d:%d]" % (start, end, step),
            )

            check(slice_tuple(t, (), (), ()), t[::], "[::]")

            check(slice_tuple(t, start, (), ()), t[start::], "[%d::]" % (start))

            check(slice_tuple(t, (), end, ()), t[:end:], "[:%d:]" % (end))

            check(
                slice_tuple(t, start, end, ()), t[start:end:], "[%d:%d:]" % (start, end)
            )

            check(slice_tuple(t, (), (), step), t[::step], "[::%d]" % (step))

            check(
                slice_tuple(t, start, (), step),
                t[start::step],
                "[%d::%d]" % (start, step),
            )

            check(slice_tuple(t, (), end, step), t[:end:step], "[:%d:%d]" % (end, step))

            check(
                slice_tuple(t, start, end, step),
                t[start:end:step],
                "[%d:%d:%d]" % (start, end, step),
            )



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
def check(want, got, mess):
    if got != want:
        print("fail: %s (want %r got %r)" % (mess, want, got))
        exit(1)


def req(x):
    return x


def default(x=0):
    return x


def req_req(x, y):
    return x - y


def req_default(x, y=42):
    return x - y


def default_default(x=63, y=42):
    return x - y


def many_args(x, y, z, a=7, b=11, c=13):
    return x * y * z * a * b * c


check(1, req(1), "pos req")
check(2, req(x=2), "named req")
check(3, default(3), "pos default")
check(4, default(x=4), "named default")

check(5, req_req(6, 1), "pos_pos req_req")
check(6, req_req(8, y=2), "pos_named req_req")
check(7, req_req(x=10, y=3), "named1_named2 req_req")
check(8, req_req(y=4, x=12), "named2_named1 req_req")

check(9, req_default(13, 4), "pos_pos req_default")
check(10, req_default(14, y=4), "pos_named req_default")
check(11, req_default(x=16, y=5), "named1_named2 req_default")
check(12, req_default(y=6, x=18), "named2_named1 req_default")
check(13, req_default(55), "pos_default req_default")
check(14, req_default(x=56), "named1_default req_default")

check(15, default_default(22, 7), "pos_pos default_default")
check(16, default_default(24, y=8), "pos_named default_default")
check(17, default_default(x=26, y=9), "named1_named2 default_default")
check(18, default_default(y=10, x=28), "named2_named1 default_default")
check(19, default_default(61), "pos_default default_default")
check(20, default_default(x=62), "named1_default default_default")
check(21, default_default(), "default_default default_default")

check(2 * 3 * 5 * 7 * 11 * 13, many_args(2, 3, 5), "many_args ()")
check(2 * 3 * 5 * 17 * 11 * 13, many_args(2, 3, 5, a=17), "many_args (a=17)")
check(2 * 3 * 5 * 17 * 19 * 13, many_args(2, 3, 5, a=17, b=19), "many_args (a=17,b=19)")
check(
    2 * 3 * 5 * 17 * 19 * 23,
    many_args(2, 3, 5, a=17, b=19, c=23),
    "many_args (a=17,b=19,c=23)",
)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#


# Foil the python3 compiler into creating separate tuples
def twople(a, b):
    return (a, b)


def cat(a, b):
    a + b


al = [1, 2]
adl = [[1, 2], [3, 4]]
bl = [1, 2]
bdl = [[1, 2], [3, 4]]
at = twople(1, 2)
adt = (twople(1, 2), twople(3, 4))
bt = twople(1, 2)
bdt = (twople(1, 2), twople(3, 4))
astr = cat("h", "ello")
bstr = cat("he", "llo")


def fail(which):
    print("fail %s" % which)
    exit(1)


# == tests elements of lists and tuples and strings
if not (al == bl):
    fail("al == bl")
if not (adl == bdl):
    fail("adl == bdl")
if not (at == bt):
    fail("at == bt")
if not (adt == bdt):
    fail("adt == bdt")
if not (astr == bstr):
    fail("astr == bstr")

# is tests for same lists/tuples, but matching string contents
if al is bl:
    fail("al is bl")
if adl is bdl:
    fail("adl is bdl")
if at is bt:
    fail("at is bt")
if adt is bdt:
    fail("adt is bdt")
if not (astr is bstr):
    fail("astr is bstr")

# in uses ==, not is

if not (al in adl):
    exit(1)
if not (bl in bdl):
    exit(1)
if not (at in adt):
    exit(1)
if not (bt in bdt):
    exit(1)



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
#
# Test math builtins
#

from math import *


def fail(which):
    print("fail %s" % which)
    exit(1)


#
# Number-theoretic and representation functions
#
if not ceil(0.5) == 1:
    fail("ceil(0.5) == 1")
if not ceil(-0.5) == 0:
    fail("ceil(-0.5) == 0")
if not copysign(1.1, -2) == -1.1:
    fail("copysign(1.1, -2) == -1.1")
if not copysign(-1.1, -2) == -1.1:
    fail("copysign(-1.1, -2) == -1.1")
if not copysign(1.1, 2) == 1.1:
    fail("copysign(1.1, 2) == 1.1")
if not copysign(-1.1, 2) == 1.1:
    fail("copysign(-1.1, 2) == 1.1")
if not fabs(-1.1) == 1.1:
    fail("fabs(-1.1) == 1.1")
if not fabs(1.1) == 1.1:
    fail("fabs(1.1) == 1.1")
if not factorial(9) == 362880:
    fail("factorial(9) == 362880")
if not floor(0.5) == 0:
    fail("floor(0.5) == 0")
if not floor(-0.5) == -1:
    fail("floor(-0.5) == -1")
if not frexp(12.5) == (0.78125, 4):
    fail("frexp(12.5) == (0.78125, 4)")
if not isclose(fsum([0.001] * 1000), 1):
    fail("isclose(fsum([0.01] * 100), 1)")
if not gcd(21, 14) == 7:
    fail("gcd(21,14) == 7")
big = 10.1
for i in range(40):
    big *= big
if not isclose(1, 1):
    fail("isclose(1,1)")
if not not isclose(1.1, 1):
    fail("not isclose(1.1,1)")
if not isclose(1.1, 1, rel_tol=0.2):
    fail("isclose(1.1,1,rel_tol=.2)")
if not isfinite(factorial(33)):
    fail("isfinite(factorial(33))")
if not not isfinite(big):
    fail("not isfinite(big)")
if not not isfinite(inf):
    fail("not isfinite(inf)")
if not isinf(inf):
    fail("isinf(inf)")
if not isinf(big):
    fail("isinf(inf)")
if not not isinf(factorial(33)):
    fail("not isinf(factorial(33))")
if not isnan(nan):
    fail("isnan(nan)")
if not isnan(inf - inf):
    fail("isnan(inf - inf)")
if not not isnan(big):
    fail("not isnan(inf)")
if not not isnan(1.0):
    fail("not isnan(1.0)")
if not ldexp(1, 2) == 4:
    fail("ldexp(1,2) == 4")
if not ldexp(1, -2) == 1 / 4:
    fail("ldexp(1,-2) == 1/4")
if not modf(1.25) == (0.25, 1.0):
    fail("modf(1.25) == (0.25, 1.0)")
if not remainder(5, 2) == 1:
    fail("remainder(5,2) == 1")
if not remainder(2.25, 0.5) == 0.25:
    fail("remainder(2.25,.5) == .25")
if not trunc(3.1) == 3:
    fail("trunc(3.1) == 3")
if not trunc(-3.1) == -3:
    fail("trunc(-3.1) == -3")
if not round(3.4) == 3:
    fail("round(3.4) == 3")
if not round(3.6) == 4:
    fail("round(3.6) == 4")
if not round(-3.4) == -3:
    fail("round(-3.4) == -3")
if not round(-3.6) == -4:
    fail("round(-3.6) == -4")
#
# Power and logarithmic functions
#
if not exp(0) == 1:
    fail("exp(0) == 1")
if not isclose(exp(1), e):
    fail("isclose(exp(1), e)")
if not isclose(exp(1) - 1, expm1(1)):
    fail("isclose(exp(1) - 1, expm1(1))")
if not isclose(log(1), 0):
    fail("isclose(log(1), 0)")
if not isclose(log(e), 1):
    fail("isclose(log(e), 1)")
if not isclose(log1p(1), log(2)):
    fail("isclose(log1p(1),log(2))")
if not isclose(log2(5.5), log(5.5) / log(2)):
    fail("isclose(log2(5.5),log(5.5)/log(2))")
if not isclose(log10(5.5), log(5.5) / log(10)):
    fail("isclose(log10(5.5),log(5.5)/log(10))")
if not isclose(pow(2.2, 3.3), exp(log(2.2) * 3.3)):
    fail("isclose(pow(2.2,3.3),exp(log(2.2) * 3.3))")
if not sqrt(4) == 2:
    fail("sqrt(4) == 2")
if not isclose(sqrt(2) * sqrt(2), 2):
    fail("isclose(sqrt(2) * sqrt(2), 2)")
#
# Trigonometric functions
#
if not isclose(acos(0), pi / 2):
    fail("isclose(acos(0), pi/2)")
if not isclose(acos(1), 0):
    fail("isclose(acos(1), 0)")
if not isclose(asin(0), 0):
    fail("isclose(asin(0), 0)")
if not isclose(asin(1), pi / 2):
    fail("isclose(asin(1), pi/2)")
if not isclose(atan(0), 0):
    fail("isclose(atan(0), 0)")
if not isclose(atan(1), pi / 4):
    fail("isclose(atan(1), pi/4)")
if not isclose(atan2(0, 1), 0):
    fail("isclose(atan2(0,1), 0)")
if not isclose(atan2(2, 2), pi / 4):
    fail("isclose(atan2(2,2), pi/4)")

if not isclose(cos(pi / 2), 0, abs_tol=1e-6):
    fail("isclose(cos(0), pi/2)")
if not isclose(cos(0), 1):
    fail("isclose(cos(1), 0)")
if not isclose(hypot(3, 4), 5):
    fail("isclose(hypot(3,4), 5)")
if not isclose(sin(0), 0):
    fail("isclose(sin(0), 0)")
if not isclose(sin(pi / 2), 1):
    fail("isclose(sin(1), pi/2)")
if not isclose(tan(0), 0):
    fail("isclose(tan(0), 0)")
if not isclose(tan(pi / 4), 1):
    fail("isclose(tan(1), pi/4)")
#
# Angular conversion
#
if not isclose(degrees(0), 0):
    fail("isclose(degrees(0),0)")
if not isclose(degrees(pi), 180):
    fail("isclose(degrees(pi),180)")
if not isclose(radians(0), 0):
    fail("isclose(radians(0),0)")
if not isclose(radians(180), pi):
    fail("isclose(radians(180), pi)")
#
# Hyperbolic functions
#
if not isclose(sinh(1), (e - 1 / e) / 2):
    fail("isclose(sinh(1), (e - 1/e) / 2)")
if not isclose(cosh(1), (e + 1 / e) / 2):
    fail("isclose(cosh(1), (e + 1/e) / 2)")
if not isclose(tanh(1), sinh(1) / cosh(1)):
    fail("isclose(tanh(1), sinh(1)/cosh(1))")
if not isclose(asinh(sinh(1)), 1):
    fail("isclose(asinh(sinh(1)), 1)")
if not isclose(acosh(cosh(1)), 1):
    fail("isclose(acosh(cosh(1)), 1)")
if not isclose(atanh(tanh(1)), 1):
    fail("isclose(atanh(tanh(1)), 1)")
#
# Special functions
#
if not isclose(erf(1), 0.8427007929497149):
    fail("isclose(erf(1), 0.8427007929497149)")
if not isclose(erfc(1), 1 - erf(1)):
    fail("isclose(erfc(1), 1-erf(1))")
if not isclose(gamma(10), factorial(9)):
    fail("isclose(gamma(10), factorial(9))")
if not isclose(lgamma(10), log(factorial(9))):
    fail("isclose(lgamma(10), log(factorial(9)))")
#
# Constants
#
if not isclose(pi * 2, tau):
    fail("isclose(pi * 2, tau)")



#
# Copyright © 2019 Keith Packard <keithp@keithp.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#


def fail(which):
    print("fail %s" % which)
    exit(1)


def check(expect, got, which):
    if expect != got:
        fail("%s: %r != %r" % (which, got, expect))


a = {1: 2, 3: 4}
b = {"a": "b", "c": "d"}
c = {1: 2, 1: 3}

check({3: 4, 1: 2}, a, "a == {3:4, 1:2}")
check({1: 3}, c, "c == {1:3}")
check({"c": "d", "a": "b"}, b, 'b = {"c":"d", "a":"b"}')

check(True, 1 in a, "1 in a")
check(False, 2 in a, "2 not in a")
check(True, 3 in a, "3 in a")
check(False, 4 in a, "4 not in a")

check(True, "a" in b, "'a' in b")
check(False, "b" in b, "'b' not in b")
check(True, "c" in b, "'c' in b")
check(False, "d" in b, "'d' not in b")

sum_a_0 = 0
sum_a_1 = 0

for i in a:
    sum_a_0 += i
    sum_a_1 += a[i]

check(4, sum_a_0, "sum a indices == 4")
check(6, sum_a_1, "sum a values == 6")

sum_b_0 = ""
sum_b_1 = ""

for i in b:
    sum_b_0 += i
    sum_b_1 += b[i]

check("ac", sum_b_0, "sum b indices == 'ac'")
check("bd", sum_b_1, "sum b values == 'bd'")

a[1] = 5
b["a"] = "e"

check(5, a[1], "a[1] == 5")
check("e", b["a"], "b['a'] == 'e'")

a[6] = 7
b["f"] = "g"

check(7, a[6], "a[6] == 7")
check("g", b["f"], "b['f'] == 'g'")

del a[1]
del b["c"]

check({3: 4, 6: 7}, a, "a = {3:4, 6:7}")
check({"a": "e", "f": "g"}, b, "b = {'a': 'e', 'f': 'g'}")
